package dk.vajhoej.jnosql.test;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.Optional;

import org.eclipse.jnosql.communication.Settings;
import org.eclipse.jnosql.communication.Value;
import org.eclipse.jnosql.communication.keyvalue.BucketManager;
import org.eclipse.jnosql.communication.keyvalue.BucketManagerFactory;
import org.eclipse.jnosql.communication.keyvalue.KeyValueConfiguration;

import dk.vajhoej.jnosql.bdbje.BDBJEConfiguration;

public class TestBDBJE {
    private static byte[] serialize(Data o) throws IOException {
        ByteBuffer bb = ByteBuffer.allocate(1000);
        bb.order(ByteOrder.LITTLE_ENDIAN);
        bb.putInt(o.getIv());
        bb.putDouble(o.getXv());
        byte[] b = o.getSv().getBytes("UTF-8");
        bb.putShort((short)b.length);
        bb.put(b);
        int n = bb.position();
        byte[] res = new byte[n];
        bb.rewind();
        bb.get(res);
        return res;
    }
    private static Data deserialize(byte[] b) throws IOException, ClassNotFoundException {
        ByteBuffer bb = ByteBuffer.wrap(b);
        bb.order(ByteOrder.LITTLE_ENDIAN);
        int iv = bb.getInt();
        double xv = bb.getDouble();
        int len = bb.getShort();
        byte[] temp = new byte[len];
        bb.get(temp);
        String sv = new String(temp, "UTF-8");
        Data res = new Data(iv, xv, sv);
        return res;
    }
    private static void dump(BucketManager bm, String key) throws ClassNotFoundException, IOException {
        System.out.printf("Key=%s\n", key);
        Optional<Value> rawvalue = bm.get(key);
        if(rawvalue.isPresent()) {
            Data value = (Data)rawvalue.get().get();
            System.out.println(value);
        } else {
            System.out.println("Not found");
        }
    }
    private static final int NREC = 1000;
    public static void main(String[] args) throws IOException, ClassNotFoundException {
        KeyValueConfiguration cfg = new BDBJEConfiguration("/work/BDB", "testdb", false, o -> serialize((Data)o), b -> deserialize(b));
        BucketManagerFactory bmf = cfg.apply(Settings.of());
        BucketManager bm = bmf.apply("testdb");
        // put data
        for(int i = 0; i < NREC; i++) {
            String key = "Key#" + (1000 + i + 1);
            Data value = new Data();
            value.setIv(i + 1);
            value.setXv(i + 1.0);
            value.setSv(String.format("This is value %d", i + 1));
            bm.put(key, value);
        }
        //
        String key;
        key = "Key#" + 1077;
        // get
        dump(bm, key);
        // delete
        bm.delete(key);
        // get non existing
        dump(bm, key);
        //
        key = "Key#" + 1088;
        // update and get
        dump(bm, key);
        Data value = (Data)bm.get(key).get().get();
        value.setIv(value.getIv() + 1);
        value.setXv(value.getXv() + 0.1);
        value.setSv(value.getSv() + " updated");
        bm.put(key, value);
        dump(bm, key);
        // list all
        // **** NOT SUPPORTED ****
        // list keys where "Key#1075" <= key < "Key#1085"
        // **** NOT SUPPORTED ****
        //
        bm.close();
        bmf.close();
    }
}
